# -*- coding: utf-8 -*-
# @Author: home-pc
# @Date:   2017-09-10 15:25:52
# @Last Modified by:   hang.zhang
# @Last Modified time: 2018-03-04 14:03:31


from scrapy_redis.queue import PriorityQueue


class easyPriorityQueue(PriorityQueue):

    pass
