# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-07-04 10:05:32
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-07-05 14:30:55
"""
    前面固定的模板，用于引用easyspider里面提供的数据库支持代码
    把spider和etl分开方便，整洁，好看
"""
import sys
reload(sys)
sys.setdefaultencoding("utf-8")
sys.path.append("../easyspider/utils")


def main():
    pass

if __name__ == '__main__':
    main()
