<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
    </head>
    <body>
    	<table>
    		<th>
    			<td>集合名</td>
    			<td>计数</td>
    			<td>检查时间</td>
    			<td>上次计数</td>
    			<td>本次新增</td>
    			<td>上次数量变更时间</td>
    		</th>
    	</table>
    </body>
</html>