# -*- coding: utf-8 -*-
# @Author: hang.zhang
# @Date:   2017-08-02 16:45:33
# @Last Modified by:   hang.zhang
# @Last Modified time: 2017-08-02 17:03:17

from scrapy.commands.crawl import Command


class easyCommand(Command):
    pass

